@@ -2,8 +2,8 @@
 #
 # sim.geno.R
 #
-# copyright (c) 2001-7, Karl W Broman
-# last modified Sep, 2007
+# copyright (c) 2001-8, Karl W Broman
+# last modified Jun, 2008
 # first written Feb, 2001
 # Licensed under the GNU General Public License version 2 (June, 1991)
 # 
@@ -61,7 +61,7 @@ function(cross, n.draws=16, step=0, off.end=0, error.prob=0.0001,
       else                              # X chromsome
         cfunc <- "sim_geno_bc"
     }
-    else if(type == "bc" || type=="riself" || type=="risib") {
+    else if(type == "bc" || type=="riself" || type=="risib" || type=="dh") {
       cfunc <- "sim_geno_bc"
       n.gen <- 2
       one.map <- TRUE
