@@ -2,9 +2,9 @@
  * 
  * scantwo_em.c
  *
- * copyright (c) 2001-6, Karl W Broman, Johns Hopkins University
+ * copyright (c) 2001-6, Karl W Broman
  *
- * last modified Oct, 2006
+ * last modified Dec, 2006
  * first written Nov, 2001
  *
  * Licensed under the GNU General Public License version 2 (June, 1991)
@@ -28,6 +28,7 @@
 #include <R_ext/PrtUtil.h>
 #include <R_ext/Applic.h>
 #include <R_ext/Linpack.h>
+#include <R_ext/Utils.h>
 #include "util.h"
 #include "scantwo_em.h"
 #define TOL 1e-12
@@ -223,6 +224,8 @@ void scantwo_1chr_em(int n_ind, int n_pos, int n_gen,
 		    i1+1, i2+1, m+1, oldllik);
 	
 	  for(r=0; r<maxit; r++) { /* loop over iterations */
+	    R_CheckUserInterrupt(); /* check for ^C */
+
 	    scantwo_em_estep(n_ind, n_gen, n_gen, Probs, Wts12, 
 			     Wts1, Wts2, Addcov, n_addcov, Intcov, 
 			     n_intcov, pheno, weights, oldparam, m, 1,
@@ -481,6 +484,8 @@ void scantwo_2chr_em(int n_ind, int n_pos1, int n_pos2, int n_gen1,
 		    i1+1, i2+1, m+1, oldllik);
 	
 	  for(r=0; r<maxit; r++) { /* loop over iterations */
+	    R_CheckUserInterrupt(); /* check for ^C */
+
 	    scantwo_em_estep(n_ind, n_gen1, n_gen2, Probs, Wts12, 
 			     Wts1, Wts2, Addcov, n_addcov, Intcov, 
 			     n_intcov, pheno, weights, oldparam, m, 1,
